<?xml version="1.0" encoding="UTF-8" ?>
<?xml-stylesheet type="text/xsl" encoding="UTF-8" ?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat"
		xmlns:md="http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat"
		xmlns:tsx="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat"
		xmlns:dc="http://purl.org/dc/elements/1.1/"
		xsi:schemaLocation="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat EATimeSeriesDataExchangeFormat.xsd http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat EAMetadata.xsd">

<!-- =================================== -->

	<dc:Title>Flag Fixer - for Wiski summary data flags (v2.1)</dc:Title>
	<dc:Date>2008-03-12</dc:Date>
	<dc:Description>XML to XML transform for the EATimeSeriesDataExchangeFormat (v1.0 to 1.2). This stylesheet adjusts the primary flag (i.e. flag1) to simplify complex 'C[GSUEM]' style Wiski flags into simple: Good, Suspect, Unchecked or Missing. XML time-series document format is otherwise maintained.</dc:Description>
	<dc:Relation/>
	<dc:Creator>Chris Beales</dc:Creator>
	<dc:Format>text/xsl</dc:Format>
	<dc:Identifier>http://websites.chrisbeales.net/TSX/tsx-Trans_Summary Mean Flag Fixer.xsl</dc:Identifier>
	<dc:Language>en</dc:Language>
	<dc:Publisher>Chris Beales, Environment Agency</dc:Publisher>
	<dc:Rights>Copyright 2006-08 Environment Agency</dc:Rights>
	<dc:Type>XSL Stylesheet</dc:Type>
	<dc:Subject>XML, EATimeSeriesDataExchangeFormat, TSX, Envrionment Agency Management System (AMS): Categorising Hydrological Data by Probability Ranking</dc:Subject>

<!-- ===================================

	Note rule base for completeness of source (15-min) data is as follows:
	
	  i) If >'minUnchecked'% unchecked, output is "Unchecked" = 4
	 ii) If >'minMissing'% missing, output is "Missing" = 5
	iii) %good = %good + %estimated (i.e. Estimated data is assumed to be Good)
	 iv) If >'minGood'% good, output is "Good" = 1
	  v) Otherwise output is "Suspect" = 2

     ...................................................... -->

	<xsl:variable name="minUnchecked">0</xsl:variable>
	<xsl:variable name="minMissing">90</xsl:variable>
	<xsl:variable name="minGood">80</xsl:variable>

<!-- ...................................................... 
	  
	- Flags are read out of a flag block, e.g.: 
		flag1="6" flag2="1" percentFlag2="90" flag3="2" percentFlag3="10"
		
		which means: Complete [ 90% Good, 10% Suspect ]
		
	- If flag2 is not between 1 and 5, flag1 will be set to "5" missing because
		is not a complex daily/monthly-mean Wiski flag block
	
	- Otherwise flag1 will be overwritten according to the rule base

   =================================== -->

	<xsl:output encoding="UTF-8" version="1.0" method="xml" indent="yes" />

	<xsl:template match="/">
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="@*|node()"> 	
		<xsl:copy>
			<xsl:apply-templates select="@*|node()" />
		</xsl:copy>
	</xsl:template>

<!-- Values -->
	<xsl:template match="tsx:Value">
		<xsl:element name="Value">
		
<!--  Fix flag1 if necessary  -->
			<xsl:if test="@flag1">
				<xsl:choose>
				<!-- If we have a primary flag (i.e. flag1) then test that flag2 is between 1 and 5 and there is a percentFlag2.... -->				
					<xsl:when test="(@flag2 &lt; 6) and @percentFlag2">
						<xsl:variable name="goodRaw">
							<xsl:choose>
								<xsl:when test="@flag2=1"><xsl:value-of select="@percentFlag2" /></xsl:when>
								<xsl:when test="@flag3=1"><xsl:value-of select="@percentFlag3" /></xsl:when>
								<xsl:when test="@flag4=1"><xsl:value-of select="@percentFlag4" /></xsl:when>
								<xsl:when test="@flag5=1"><xsl:value-of select="@percentFlag5" /></xsl:when>
								<xsl:when test="@flag6=1"><xsl:value-of select="@percentFlag6" /></xsl:when>
							</xsl:choose>
						</xsl:variable>

						<xsl:variable name="estimated">
							<xsl:choose>
								<xsl:when test="@flag2=3"><xsl:value-of select="@percentFlag2" /></xsl:when>
								<xsl:when test="@flag3=3"><xsl:value-of select="@percentFlag3" /></xsl:when>
								<xsl:when test="@flag4=3"><xsl:value-of select="@percentFlag4" /></xsl:when>
								<xsl:when test="@flag5=3"><xsl:value-of select="@percentFlag5" /></xsl:when>
								<xsl:when test="@flag6=3"><xsl:value-of select="@percentFlag6" /></xsl:when>
							</xsl:choose>
						</xsl:variable>

						<xsl:variable name="good0">
							<xsl:if test="$estimated"><xsl:value-of select="$estimated" /></xsl:if>
							<xsl:if test="$estimated=''">0</xsl:if>
						</xsl:variable>								

						<xsl:variable name="good1">
							<xsl:if test="$goodRaw"><xsl:value-of select="$goodRaw" /></xsl:if>
							<xsl:if test="$goodRaw=''">0</xsl:if>
						</xsl:variable>								

						<xsl:variable name="good"><xsl:value-of select="$good0 + $good1" /></xsl:variable>								

						<xsl:variable name="unchecked">
							<xsl:choose>
								<xsl:when test="@flag2=4"><xsl:value-of select="@percentFlag2" /></xsl:when>
								<xsl:when test="@flag3=4"><xsl:value-of select="@percentFlag3" /></xsl:when>
								<xsl:when test="@flag4=4"><xsl:value-of select="@percentFlag4" /></xsl:when>
								<xsl:when test="@flag5=4"><xsl:value-of select="@percentFlag5" /></xsl:when>
								<xsl:when test="@flag6=4"><xsl:value-of select="@percentFlag6" /></xsl:when>
							</xsl:choose>
						</xsl:variable>

						<xsl:variable name="missing">
							<xsl:choose>
								<xsl:when test="@flag2=5"><xsl:value-of select="@percentFlag2" /></xsl:when>
								<xsl:when test="@flag3=5"><xsl:value-of select="@percentFlag3" /></xsl:when>
								<xsl:when test="@flag4=5"><xsl:value-of select="@percentFlag4" /></xsl:when>
								<xsl:when test="@flag5=5"><xsl:value-of select="@percentFlag5" /></xsl:when>
								<xsl:when test="@flag6=5"><xsl:value-of select="@percentFlag6" /></xsl:when>
							</xsl:choose>
						</xsl:variable>

					<!-- Check output using rule base ... -->
						<xsl:choose>
							<xsl:when test="$unchecked > $minUnchecked">
								<xsl:attribute name="flag1">4</xsl:attribute>
							</xsl:when>

							<xsl:otherwise>
								<xsl:choose>
									<xsl:when test="$missing > $minMissing">
										<xsl:attribute name="flag1">5</xsl:attribute>
									</xsl:when>

									<xsl:otherwise>
										<xsl:choose>
											<xsl:when test="$good > $minGood">
												<xsl:attribute name="flag1">1</xsl:attribute>
											</xsl:when>

											<xsl:otherwise>
											<!-- Suspect -->
												<xsl:attribute name="flag1">2</xsl:attribute>
											</xsl:otherwise>
										</xsl:choose>

									</xsl:otherwise>
								</xsl:choose>

							</xsl:otherwise>
						</xsl:choose>

					</xsl:when> <!-- (flag2 < 6) and %flag2 exists -->

				<!-- ...not Wiski daily-mean flag block so write flag1 as is -->						
					<xsl:otherwise>
						<xsl:copy-of select="@flag1" />
					</xsl:otherwise>     
				</xsl:choose>    
			</xsl:if>

<!--  Copy remaning attributes and the value -->
			<xsl:for-each select="attribute::node()[name() != 'flag1']">
				<xsl:attribute name="{name()}">
					<xsl:value-of select="." />
				</xsl:attribute>
			</xsl:for-each>

			<xsl:value-of select="."/>

		</xsl:element>
	</xsl:template>

</xsl:stylesheet>

